/*
 *  TOPPERS/ASP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Advanced Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2012 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 */


#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "kernel_cfg.h"
#include "sample1.h"

#include "app_param.h"

void  pico_gpio_led_set( int on );
int  pico_gpio_btn1( void );
int  pico_gpio_btn2( void );

int  fat_test_init( void );
int  fat_test_read( char *filename, char *buff, int bsize );

unsigned char  *read_fontx2_a( unsigned int code );
unsigned char  *read_fontx2_k( unsigned int code );


char  *def_img_filename[ DEF_DISP_FILE_NUM ] = {
    "image1.bmp",
    "image2.bmp",
    "image3.bmp",
    "image4.bmp",
    "image5.bmp",
    "image6.bmp",
    "image7.bmp",
    "image8.bmp"
};

char  *def_txt_filename[ DEF_DISP_FILE_NUM ] = {
    "text1.txt",
    "text2.txt",
    "text3.txt",
    "text4.txt",
    "text5.txt",
    "text6.txt",
    "text7.txt",
    "text8.txt"
};

int  index_file_num;


char  text_msg_buffer[ DEF_TEXT_READ ];

uint16_t  text_img_buffer[ DEF_FONT_HIGH ][ DEF_TEXT_BUFF ];
int  text_img_pos;
int  text_scroll;

extern  uint16_t  image_buffer[];


void delay( int ms )
{
  tslp_tsk( ms );
}


void  set_font( uint8_t *font, uint16_t *buff, int pos, int width, uint16_t color )
{
    int  i, j, k;
    int  row;
    int  w = (width/8);   /* font width byte */
    uint8_t  pat;

    /* row */
    for( i=0 ; i<DEF_FONT_HIGH ; i++ ) {
        row = DEF_TEXT_BUFF * i;
        /* col */
        for( j=0 ; j<w ; j++ ) {
            pat = 0x80;
            for( k=0 ; k<8 ; k++ ) {
                if( (font[ i * w + j ] & pat) != 0 ) {
                    buff[ row + pos + j*8 + k ] = color;
                }
                pat >>= 1; /* bit shift */
            }
        }
    }
}


void  clear_text_img_buffer( void )
{
    int  i;
    uint16_t  *dst;

    dst = text_img_buffer;
    for( i=0 ; i<(DEF_FONT_HIGH*DEF_TEXT_BUFF) ; i++ ) {
        *dst++ = 0x0000;
    }
}


int  set_message( unsigned int col, unsigned char *strbuff, unsigned int size )
{
    unsigned int  pos, bitpos;
    unsigned char  *fontdata;
    unsigned int  code;
    uint16_t  *base;
    uint16_t  color;

    pos = 0 ;
    bitpos = col ;
    base = (uint16_t *)text_img_buffer;

    while( pos < size ) {
        color = 0xffff;   /* white */

        /* get 1 char */
        code = strbuff[ pos ] ;  /* get 1st byte */

        /* check half/full */
        if( code < 0x80 ) {
            /* half(8dot) */
            if( code == 0x0d || code == 0x0a ) {
                code = ' ' ;
            }
            fontdata = read_fontx2_a( code );
            set_font( fontdata, base, bitpos, DEF_FONT_WIDTH/2, color );
            bitpos += DEF_FONT_WIDTH/2 ;
            pos++ ;

        } else {
            /* full(16dot) */
            code = (code<<8) + strbuff[ pos+1 ];  /* get 2nd byte and marge */
            fontdata = read_fontx2_k( code );
            set_font( fontdata, base, bitpos, DEF_FONT_WIDTH, color );
            bitpos += DEF_FONT_WIDTH;
            pos += 2;
        }

        if( bitpos > (DEF_IMG_WIDTH + (DEF_TEXT_NUM - 1)*DEF_FONT_WIDTH) ) {
           /* limit of text buffer at display position */
           break ;
        }
    }

    return  bitpos;
}


int hub75_test_init( void );
int hub75_test_loop( void );


void task1(intptr_t exinf)
{
	ER  er;
	FLGPTN  ptn;

	er = wai_flg( FLG1, 0x02, TWF_ORW, &ptn );
	er = clr_flg( FLG1, ~0x02 );

	while( true ) {
		tslp_tsk( 5 );
		text_img_pos++;
		if( text_img_pos > text_scroll ) {
			er = set_flg( FLG1, 0x01 );

			er = wai_flg( FLG1, 0x02, TWF_ORW, &ptn );
			er = clr_flg( FLG1, ~0x02 );
		}
	}
}


void  display_data_update( void )
{
	int  ret;

	ret = fat_test_read( def_img_filename[index_file_num], (char *)image_buffer, DEF_IMAGEBUFF );

	ret = fat_test_read( def_txt_filename[index_file_num], (char *)text_msg_buffer, DEF_TEXT_READ );
	syslog(LOG_NOTICE, "READ: %s (%dbyte)", def_txt_filename[index_file_num], ret );
	clear_text_img_buffer();
	text_img_pos = 0;
	text_scroll = set_message( DEF_IMG_WIDTH, (unsigned char *)text_msg_buffer, ret );

	if( ++index_file_num >= DEF_DISP_FILE_NUM ) {
		index_file_num = 0;
	}
}


void task2(intptr_t exinf)
{
	int  ret;
	ER  er;
	FLGPTN  ptn;

	display_data_update();
	er = set_flg( FLG1, 0x02 );

	while( true ) {
		er = wai_flg( FLG1, 0x01, TWF_ORW, &ptn );
		er = clr_flg( FLG1, ~0x01 );
		display_data_update();
		er = set_flg( FLG1, 0x02 );
	}
}


void tex_routine(TEXPTN texptn, intptr_t exinf)
{
}


#ifdef CPUEXC1
void
cpuexc_handler(void *p_excinf)
{
}
#endif /* CPUEXC1 */


void main_task(intptr_t exinf)
{
	int  i;
	uint16_t  *dst;
	ER  er;

	index_file_num = 0;
	fat_test_init();

	text_img_pos = 0;
	text_scroll = 0;

	/* clear text image buffer */
	dst = (uint16_t *)text_img_buffer;
	for( i=0 ; i<(DEF_FONT_HIGH * DEF_TEXT_BUFF) ; i++ ) {
		*dst++ = 0x0000;
	}

	(void)hub75_test_init();

	er = clr_flg( FLG1, 0x00 );

	act_tsk(TASK1);
	act_tsk(TASK2);

	while( 1 ) {
		(void)hub75_test_loop();
		tslp_tsk( 4 );
	}
}
